;Firmware for a DDR mat controller, using touchless switch technology.
;See http://sprite.student.utwente.nl/~jeroen/projects/ddr-mat/ for more
;info. This software is licensed under the GNU GPL. No implicit or other
;kind of warranty to the workings of this program is provided.

;If you use this sourcecode to implement something of your own, I'd love to hear
;about it. I can be reached at jeroen@ietsmet.nl .

.device AT90S2313

;includes
;.include "at2313.inc"

;  -----Config----
;These variables should be OK most of the time.

;Sensitivity: if this number is smaller, the pads will become more sensitive
;(could mean they already detect your foot when it isn't even on the pad yet)
.equ SENS	=	25

;Jitter: when this amount is bigger, the pads will more often recalibrate them-
;selves (they'll 'think' there's no-one on them more easily). Make this higher
;if your pads won't calibrate as soon as the power turns on. Lower this when,
;during play, your pads recalibrate themselves. Don't set this to more than 
;2 times the value of SENS.
.equ JITTER	=	35


;  ----Config end---

;Port declarations

;PortB
.equ pad1	=	7
.equ pad2	=	6
.equ pad3	=	5
.equ pad4	=	4
.equ led4	=	3
.equ led3	=	2
.equ led2	=	1
.equ led1	=	0

;Port D
.equ btn	=	5
.equ game4	=	3
.equ game3	=	2
.equ game2	=	1
.equ game1	=	0

.def cnt1	=	r1
.def cnt2	=	r2
.def cnt3	=	r3
.def cnt4	=	r4
.def nlow1	=	r5
.def nlow2	=	r6
.def nlow3	=	r7
.def nlow4	=	r8
.def temp	=	R16 	;The temp register
.def temp2	=	R17 	;Second temp register
.def temp3	=	R18
;Registers to store the average on 64 measurements
.def avg1l	=	R19
.def avg2l	=	R20
.def avg3l	=	R21
.def avg4l	=	R22
.def avg1h	=	R23
.def avg2h	=	R24
.def avg3h	=	R25
.def avg4h	=	R26
;Which leds are on
.def leds	=	r27
;Low-marks
.def low1	=	r28
.def low2	=	r29
.def low3	=	r30
.def low4	=	r31


;The code itself
;	.cseg			; switch to Code Segment
;	.org	0		
	rjmp start		; reset vector
	reti			; External Int 0 not enabled
	reti			; External Int 1 not enabled
	reti			; Timer 1 Capture Int not enabled
	reti			; Timer 1 Compare match A Int not enabled
	reti			; Timer 1 Overflow Int not enabled
	reti			; Timer 0 Overflow Int not enabled
	reti			; UART ReceiveInt
	reti			; UART Date Reg Empty Int not enabled
	reti			; UART tx'ed Int Not enabled
	reti			; Anacomp interrupt Not enabled

start:
	ldi temp,RAMEND		; top of memory
	out SPL,temp		; init stack pointer
; - Setup Port B . . .
	ldi temp,0b001111	; Port B direction options
	out DDRB,temp		; Setup port direction
	ldi temp,0		; Port B output values
	out PORTB,temp		; Setup port control options
; - Setup Port D . . .
	ldi temp,0b001111	; Port D direction options
	out DDRD,temp		; Setup port direction
	ldi temp,0b101111	; Port D output values
	out PORTD,temp		; Setup port control options

	ldi low1,0
	ldi low2,0
	ldi low3,0
	ldi low4,0

	;This is the main loop
vast:
;Zero out hit counters
	ldi avg1l,0
	ldi avg2l,0
	ldi avg3l,0
	ldi avg4l,0
	ldi avg1h,0
	ldi avg2h,0
	ldi avg3h,0
	ldi avg4h,0

	
;Measure all pads 64 times.
	ldi temp3,64
loop1:
	rcall detect1
	rcall detect2
	rcall detect3
	rcall detect4
	dec temp3
	brne loop1
	
	;Divide everything by 4 so that a number from 0 to 255 remains in
	;avgNl. This number indicates the 'amount of mass' on the pads.
	lsr avg1h
	ror avg1l	
	lsr avg1h
	ror avg1l	
	lsr avg2h
	ror avg2l	
	lsr avg2h
	ror avg2l	
	lsr avg3h
	ror avg3l	
	lsr avg3h
	ror avg3l	
	lsr avg4h
	ror avg4l	
	lsr avg4h
	ror avg4l	


	ldi leds,0
	
	sbic pind,btn ;Button pressed?
	rjmp nietcalbtn
	;Yes. Reset low marks to help setting the variable resistors.
	ldi low1,110
	ldi low2,110
	ldi low3,110
	ldi low4,110
nietcalbtn:


;See if any leds should go on
	cp avg1l,low1
	brlo nieton1
	sbr leds,1
nieton1:

	cp avg2l,low2
	brlo nieton2
	sbr leds,2
nieton2:

	cp avg3l,low3
	brlo nieton3
	sbr leds,4
nieton3:

	cp avg4l,low4
	brlo nieton4
	sbr leds,8
nieton4:

	
;These routines do the auto-calibrating part: if an avgNl variable stays 
;constant for 256 cycles (where 'constant' means it can have a deviation of
;at most JITTER), that value will become the new low-mark. The value of SENS
;is added to it, and the routines above this text will send a 'button pressed'
;to the PC as soon as the value of avgNl exceeds this.

	mov temp,avg1l
	sub temp,nlow1
	subi temp,-(JITTER/2)
	cpi temp,JITTER
	brsh calreset1
	inc cnt1
	brne caldone1
	mov low1,nlow1
	subi low1,-SENS
calreset1:
	clr cnt1
	mov nlow1,avg1l
caldone1:

	mov temp,avg2l
	sub temp,nlow2
	subi temp,-(JITTER/2)
	cpi temp,JITTER
	brsh calreset2
	inc cnt2
	brne caldone2
	mov low2,nlow2
	subi low2,-SENS
calreset2:
	clr cnt2
	mov nlow2,avg2l
caldone2:

	mov temp,avg3l
	sub temp,nlow3
	subi temp,-(JITTER/2)
	cpi temp,JITTER
	brsh calreset3
	inc cnt3
	brne caldone3
	mov low3,nlow3
	subi low3,-SENS
calreset3:
	clr cnt3
	mov nlow3,avg3l
caldone3:

	mov temp,avg4l
	sub temp,nlow4
	subi temp,-(JITTER/2)
	cpi temp,JITTER
	brsh calreset4
	inc cnt4
	brne caldone4
	mov low4,nlow4
	subi low4,-SENS
calreset4:
	clr cnt4
	mov nlow4,avg4l
caldone4:


;Send the detected values to the gameport, but invert them first. The button-
;inputs are active-low.
	push leds
	com leds
	in temp,portd
	andi leds,0x0f
	andi temp,0xf0
	or temp,leds
	out portd,temp
	pop leds
	rjmp vast	


;The detectX-routines work like this:
;First of all the pin is grounded, so the pad will get flooded with electrons.
;If there is a foot above the pad, the electrons will induce a shift of 
;electrons in your body too. After waiting some time for this process to 
;complete (and, meanwhile, turning off the leds so they can't disturb the 
;measuring process) the pin is tristated and the time needed for the
;50K potmeter to pump all the electrons out again is measured. Because
;there's a bigger energy shift when your foot is above a pad, the time
;needed will be bigger then. This is compared to a running average, if the
;value is more than the average +  a few % then the detection counts as a hit
;and increases ctr1.

detect1:
	cbi portb,pad1
    	sbi ddrb,pad1
	rcall wacht
	ldi temp,0
	cbi ddrb,pad1
	ldi temp,0
	sbis pinb,pad1
	inc temp
	sbis pinb,pad1
	inc temp
	sbis pinb,pad1
	inc temp
	sbis pinb,pad1
	inc temp
	sbis pinb,pad1
	inc temp
	sbis pinb,pad1
	inc temp
	sbis pinb,pad1
	inc temp
	sbis pinb,pad1
	inc temp
	sbis pinb,pad1
	inc temp
	sbis pinb,pad1
	inc temp
	sbis pinb,pad1
	inc temp
	sbis pinb,pad1
	inc temp
	sbis pinb,pad1
	inc temp
	sbis pinb,pad1
	inc temp
	sbis pinb,pad1
	inc temp
	sbis pinb,pad1
	inc temp
	sbis pinb,pad1
	inc temp
	sbis pinb,pad1
	inc temp
	sbis pinb,pad1
	inc temp
	sbis pinb,pad1
	inc temp
	sbis pinb,pad1
	inc temp
	sbis pinb,pad1
	inc temp
	sbis pinb,pad1
	inc temp
	sbis pinb,pad1
	inc temp
	sbis pinb,pad1
	inc temp
	sbis pinb,pad1
	inc temp
	sbis pinb,pad1
	inc temp
	sbis pinb,pad1
	inc temp
	sbis pinb,pad1
	inc temp
	sbis pinb,pad1
	inc temp
	sbis pinb,pad1
	inc temp
	add avg1l,temp
	ldi temp,0
	adc avg1h,temp
	ret



detect2:
	ldi temp,0
	cbi portb,pad2
    	sbi ddrb,pad2
	rcall wacht
	cbi ddrb,pad2
	ldi temp,0
	sbis pinb,pad2
	inc temp
	sbis pinb,pad2
	inc temp
	sbis pinb,pad2
	inc temp
	sbis pinb,pad2
	inc temp
	sbis pinb,pad2
	inc temp
	sbis pinb,pad2
	inc temp
	sbis pinb,pad2
	inc temp
	sbis pinb,pad2
	inc temp
	sbis pinb,pad2
	inc temp
	sbis pinb,pad2
	inc temp
	sbis pinb,pad2
	inc temp
	sbis pinb,pad2
	inc temp
	sbis pinb,pad2
	inc temp
	sbis pinb,pad2
	inc temp
	sbis pinb,pad2
	inc temp
	sbis pinb,pad2
	inc temp
	sbis pinb,pad2
	inc temp
	sbis pinb,pad2
	inc temp
	sbis pinb,pad2
	inc temp
	sbis pinb,pad2
	inc temp
	sbis pinb,pad2
	inc temp
	sbis pinb,pad2
	inc temp
	sbis pinb,pad2
	inc temp
	sbis pinb,pad2
	inc temp
	sbis pinb,pad2
	inc temp
	sbis pinb,pad2
	inc temp
	sbis pinb,pad2
	inc temp
	sbis pinb,pad2
	inc temp
	sbis pinb,pad2
	inc temp
	sbis pinb,pad2
	inc temp
	sbis pinb,pad2
	inc temp
	add avg2l,temp
	ldi temp,0
	adc avg2h,temp
	
	ret



;This routine is called whenever there needs to be waited some time for
;the electrons to flow in the pad.
wacht:
	ldi temp,0
	;Show led values
	andi leds,$f
	out portb,leds
	ldi temp,128
wacht1:
	dec temp
	brne wacht1
	;Clear leds so they won't influence the measuring process
	out portb,temp
	ldi temp,128
wacht2:
	dec temp
	brne wacht2
	ret


detect3:
	ldi temp,0
	cbi portb,pad3
    	sbi ddrb,pad3
	rcall wacht
	cbi ddrb,pad3
	ldi temp,0
	sbis pinb,pad3
	inc temp
	sbis pinb,pad3
	inc temp
	sbis pinb,pad3
	inc temp
	sbis pinb,pad3
	inc temp
	sbis pinb,pad3
	inc temp
	sbis pinb,pad3
	inc temp
	sbis pinb,pad3
	inc temp
	sbis pinb,pad3
	inc temp
	sbis pinb,pad3
	inc temp
	sbis pinb,pad3
	inc temp
	sbis pinb,pad3
	inc temp
	sbis pinb,pad3
	inc temp
	sbis pinb,pad3
	inc temp
	sbis pinb,pad3
	inc temp
	sbis pinb,pad3
	inc temp
	sbis pinb,pad3
	inc temp
	sbis pinb,pad3
	inc temp
	sbis pinb,pad3
	inc temp
	sbis pinb,pad3
	inc temp
	sbis pinb,pad3
	inc temp
	sbis pinb,pad3
	inc temp
	sbis pinb,pad3
	inc temp
	sbis pinb,pad3
	inc temp
	sbis pinb,pad3
	inc temp
	sbis pinb,pad3
	inc temp
	sbis pinb,pad3
	inc temp
	sbis pinb,pad3
	inc temp
	sbis pinb,pad3
	inc temp
	sbis pinb,pad3
	inc temp
	sbis pinb,pad3
	inc temp
	sbis pinb,pad3
	inc temp
	
	add avg3l,temp
	ldi temp,0
	adc avg3h,temp
	ret


detect4:
	ldi temp,0
	cbi portb,pad4
    	sbi ddrb,pad4
	rcall wacht
	cbi ddrb,pad4
	ldi temp,0
	sbis pinb,pad4
	inc temp
	sbis pinb,pad4
	inc temp
	sbis pinb,pad4
	inc temp
	sbis pinb,pad4
	inc temp
	sbis pinb,pad4
	inc temp
	sbis pinb,pad4
	inc temp
	sbis pinb,pad4
	inc temp
	sbis pinb,pad4
	inc temp
	sbis pinb,pad4
	inc temp
	sbis pinb,pad4
	inc temp
	sbis pinb,pad4
	inc temp
	sbis pinb,pad4
	inc temp
	sbis pinb,pad4
	inc temp
	sbis pinb,pad4
	inc temp
	sbis pinb,pad4
	inc temp
	sbis pinb,pad4
	inc temp
	sbis pinb,pad4
	inc temp
	sbis pinb,pad4
	inc temp
	sbis pinb,pad4
	inc temp
	sbis pinb,pad4
	inc temp
	sbis pinb,pad4
	inc temp
	sbis pinb,pad4
	inc temp
	sbis pinb,pad4
	inc temp
	sbis pinb,pad4
	inc temp
	sbis pinb,pad4
	inc temp
	sbis pinb,pad4
	inc temp
	sbis pinb,pad4
	inc temp
	sbis pinb,pad4
	inc temp
	sbis pinb,pad4
	inc temp
	sbis pinb,pad4
	inc temp
	sbis pinb,pad4
	inc temp
	add avg4l,temp
	ldi temp,0
	adc avg4h,temp
	ret

